@@ -12,7 +12,9 @@ Implementation
 import os
 import sys
 import string
-from random import choice, random
+import mimetypes
+from itertools import repeat, chain
+from random import random
 
 try:
     from pulsar.utils.pep import ispy3k, range
@@ -20,15 +22,18 @@ except ImportError:     # pragma    nocover
     sys.path.append('../../')
     from pulsar.utils.pep import ispy3k, range
 
-from pulsar import (HttpRedirect, HttpException, version, JAPANESE,
-                    coroutine_return)
-from pulsar.utils.httpurl import Headers, ENCODE_URL_METHODS
+from pulsar import HttpRedirect, HttpException, version, JAPANESE, CHINESE
+from pulsar.utils.httpurl import (Headers, ENCODE_URL_METHODS,
+                                  ENCODE_BODY_METHODS)
 from pulsar.utils.html import escape
 from pulsar.apps import wsgi, ws
-from pulsar.apps.wsgi import route, Html, Json, HtmlDocument, GZipMiddleware
+from pulsar.apps.wsgi import (route, Html, Json, HtmlDocument, GZipMiddleware,
+                              AsyncString)
 from pulsar.utils.structures import MultiValueDict
 from pulsar.utils.system import json
 
+METHODS = frozenset(chain((m.lower() for m in ENCODE_URL_METHODS),
+                          (m.lower() for m in ENCODE_BODY_METHODS)))
 pyversion = '.'.join(map(str, sys.version_info[:3]))
 ASSET_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'assets')
 FAVICON = os.path.join(ASSET_DIR, 'favicon.ico')
@@ -39,54 +44,114 @@ else:   # pragma nocover
     characters = string.letters + string.digits
 
 
-def template():
-    name = os.path.join(ASSET_DIR, 'template.html')
-    with open(name, 'r') as file:
-        return file.read()
+def asset(name, mode='r', chunk_size=None):
+    name = os.path.join(ASSET_DIR, name)
+    if os.path.isfile(name):
+        content_type, encoding = mimetypes.guess_type(name)
+        if chunk_size:
+            def _chunks():
+                with open(name, mode) as file:
+                    while True:
+                        data = file.read(chunk_size)
+                        if not data:
+                            break
+                        yield data
+            data = _chunks()
+        else:
+            with open(name, mode) as file:
+                data = file.read()
+        return data, content_type, encoding
 
 
-class HttpBin(wsgi.Router):
-
+class BaseRouter(wsgi.Router):
+    ########################################################################
+    #    INTERNALS
     def bind_server_event(self, request, event, handler):
         consumer = request.environ['pulsar.connection'].current_consumer()
         consumer.bind_event(event, handler)
 
+    def info_data_response(self, request, **params):
+        data = self.info_data(request, **params)
+        return Json(data).http_response(request)
+
+    def info_data(self, request, **params):
+        headers = self.getheaders(request)
+        data = {'method': request.method,
+                'headers': headers,
+                'pulsar': self.pulsar_info(request)}
+        if request.method in ENCODE_URL_METHODS:
+            data['args'] = dict(request.url_data)
+        else:
+            args, files = request.data_and_files()
+            jfiles = MultiValueDict()
+            for name, parts in files.lists():
+                for part in parts:
+                    try:
+                        part = part.string()
+                    except UnicodeError:
+                        part = part.base64()
+                    jfiles[name] = part
+            data.update((('args', dict(args)),
+                         ('files', dict(jfiles))))
+        data.update(params)
+        return data
+
+    def getheaders(self, request):
+        headers = Headers(kind='client')
+        for k in request.environ:
+            if k.startswith('HTTP_'):
+                headers[k[5:].replace('_', '-')] = request.environ[k]
+        return dict(headers)
+
+    def pulsar_info(self, request):
+        return request.get('pulsar.connection').info()
+
+
+class HttpBin(BaseRouter):
+    '''The main :class:`.Router` for the HttpBin application
+    '''
     def get(self, request):
         '''The home page of this router'''
         ul = Html('ul')
         for router in sorted(self.routes, key=lambda r: r.creation_count):
             a = router.link(escape(router.route.path))
-            li = Html('li', a, ' %s' % router.parameters.get('title', ''))
+            a.addClass(router.name)
+            for method in METHODS:
+                if router.getparam(method):
+                    a.addClass(method)
+            li = Html('li', a, ' %s' % router.getparam('title', ''))
             ul.append(li)
-        title = 'Pulsar HttpBin'
+        title = 'Pulsar'
         html = request.html_document
         html.head.title = title
-        html.head.links.append('/media/httpbin.css')
-        html.head.scripts.append('//code.jquery.com/jquery.min.js')
-        html.head.scripts.append('/media/httpbin.js')
+        html.head.links.append('httpbin.css')
+        html.head.links.append('favicon.ico', rel="icon", type='image/x-icon')
+        html.head.scripts.append('jquery')
+        html.head.scripts.append('httpbin.js')
         ul = ul.render(request)
-        body = template() % (title, version, ul, pyversion, JAPANESE)
+        templ, _, _ = asset('template.html')
+        body = templ % (title, JAPANESE, CHINESE, version, pyversion, ul)
         html.body.append(body)
         return html.http_response(request)
 
-    @route('get', title='Returns GET data')
-    def _get(self, request):
+    @route(title='Returns GET data')
+    def get_get(self, request):
         return self.info_data_response(request)
 
-    @route('post', method='post', title='Returns POST data')
-    def _post(self, request):
+    @route(title='Returns POST data')
+    def post_post(self, request):
         return self.info_data_response(request)
 
-    @route('patch', method='patch', title='Returns PATCH data')
-    def _patch(self, request):
+    @route(title='Returns PATCH data')
+    def patch_patch(self, request):
         return self.info_data_response(request)
 
-    @route('put', method='put', title='Returns PUT data')
-    def _put(self, request):
+    @route(title='Returns PUT data')
+    def put_put(self, request):
         return self.info_data_response(request)
 
-    @route('delete', method='delete', title='Returns DELETE data')
-    def _delete(self, request):
+    @route(title='Returns DELETE data')
+    def delete_delete(self, request):
         return self.info_data_response(request)
 
     @route('redirect/<int(min=1,max=10):times>', defaults={'times': 5},
@@ -102,16 +167,15 @@ class HttpBin(wsgi.Router):
            title='Returns a preset size of data (limit at 8MB)')
     def getsize(self, request):
         size = request.urlargs['size']
-        data = {'size': size,
-                'data': ''.join(('d' for n in range(size)))}
+        data = {'size': size, 'data': 'd' * size}
         return self.info_data_response(request, **data)
 
-    @route('gzip', title='Returns gzip encoded data')
+    @route(title='Returns gzip encoded data')
     def gzip(self, request):
-        response = yield self.info_data_response(request, gzipped=True)
-        coroutine_return(GZipMiddleware(10)(request.environ, response))
+        response = self.info_data_response(request, gzipped=True)
+        return GZipMiddleware(10)(request.environ, response)
 
-    @route('cookies', title='Returns cookie data')
+    @route(title='Returns cookie data')
     def cookies(self, request):
         cookies = request.cookies
         d = dict(((c.key, c.value) for c in cookies.values()))
@@ -134,7 +198,7 @@ class HttpBin(wsgi.Router):
         request.response.content_type = 'text/html'
         raise HttpException(status=request.urlargs['status'])
 
-    @route('response-headers', title='Returns response headers')
+    @route(title='Returns response headers')
     def response_headers(self, request):
         class Gen:
             headers = None
@@ -181,16 +245,12 @@ class HttpBin(wsgi.Router):
     def request_stream(self, request):
         m = request.urlargs['m']
         n = request.urlargs['n']
-        if m*n > 8388608:
-            # limit at 8 megabytes of total data
-            raise HttpException(status=403)
-        stream = ('Chunk %s\n%s\n\n' % (i+1, ''.join((
-            choice(characters) for _ in range(m)))) for i in range(n))
-        request.response.content = stream
+        request.response.content_type = 'text/plain'
+        request.response.content = repeat(b'a' * m, n)
         return request.response
 
-    @route('websocket', title='A web socket graph')
-    def request_websocket(self, request):
+    @route(title='A web socket graph')
+    def websocket(self, request):
         data = open(os.path.join(os.path.dirname(__file__),
                                  'assets', 'websocket.html')).read()
         scheme = 'wss' if request.is_secure else 'ws'
@@ -200,8 +260,8 @@ class HttpBin(wsgi.Router):
         request.response.content = data
         return request.response
 
-    @route('stats', title='Live server statistics')
-    def request_stats(self, request):
+    @route(title='Live server statistics')
+    def stats(self, request):
         '''Live stats for the server.
 
         Try sending lots of requests
@@ -213,62 +273,37 @@ class HttpBin(wsgi.Router):
         # docs.head.scripts
         return doc.http_response(request)
 
-    @route('expect', method='post', title='Expectation Failed')
-    def expectation_failure(self, request):
-        stream = request.get('wsgi.input')
-        stream.fail()
-        return self.info_data_response(request)
+    @route('clip/<int(min=256,max=16777216):chunk_size>',
+           defaults={'chunk_size': 4096},
+           title='Show a video clip')
+    def clip(self, request):
+        c = request.urlargs['chunk_size']
+        data, ct, encoding = asset('clip.mp4', 'rb', chunk_size=c)
+        response = request.response
+        response.content_type = ct
+        response.encoding = encoding
+        response.content = data
+        return response
 
     ########################################################################
     #    BENCHMARK ROUTES
-    @route('json')
-    def bench_json(self, request):
+    @route()
+    def json(self, request):
         return Json({'message': "Hello, World!"}).http_response(request)
 
-    @route('plaintext')
-    def bench_text(self, request):
-        r = request.response
-        r.content = 'Hello, World!'
-        r.content_type = 'text/plain; charset=utf-8'
-        return r
+    @route()
+    def plaintext(self, request):
+        return AsyncString('Hello, World!').http_response(request)
 
-    ########################################################################
-    #    INTERNALS
-    def info_data_response(self, request, **params):
-        data = self.info_data(request, **params)
-        return Json(data).http_response(request)
 
-    def info_data(self, request, **params):
-        headers = self.getheaders(request)
-        data = {'method': request.method,
-                'headers': headers,
-                'pulsar': self.pulsar_info(request)}
-        if request.method in ENCODE_URL_METHODS:
-            data['args'] = dict(request.url_data)
+class ExpectFail(BaseRouter):
+
+    def post(self, request):
+        chunk = request.get('wsgi.input')
+        if not chunk.done():
+            chunk.fail()
         else:
-            args, files = yield request.data_and_files()
-            jfiles = MultiValueDict()
-            for name, parts in files.lists():
-                for part in parts:
-                    try:
-                        part = part.string()
-                    except UnicodeError:
-                        part = part.base64()
-                    jfiles[name] = part
-            data.update((('args', dict(args)),
-                         ('files', dict(jfiles))))
-        data.update(params)
-        coroutine_return(data)
-
-    def getheaders(self, request):
-        headers = Headers(kind='client')
-        for k in request.environ:
-            if k.startswith('HTTP_'):
-                headers[k[5:].replace('_', '-')] = request.environ[k]
-        return dict(headers)
-
-    def pulsar_info(self, request):
-        return request.get('pulsar.connection').info()
+            return self.info_data_response(request)
 
 
 class Graph(ws.WS):
@@ -281,9 +316,10 @@ class Site(wsgi.LazyWsgi):
 
     def setup(self, environ):
         router = HttpBin('/')
-        return wsgi.WsgiHandler([wsgi.clean_path_middleware,
+        return wsgi.WsgiHandler([ExpectFail('expect'),
+                                 wsgi.wait_for_body_middleware,
+                                 wsgi.clean_path_middleware,
                                  wsgi.authorization_middleware,
-                                 wsgi.FileRouter('/favicon.ico', FAVICON),
                                  wsgi.MediaRouter('media', ASSET_DIR,
                                                   show_indexes=True),
                                  ws.WebSocket('/graph-data', Graph()),
