@@ -11,7 +11,7 @@ from .manage import server
 
 class Echo(WS):
 
-    def __init__(self, loop=None):
+    def __init__(self, loop):
         self.queue = asyncio.Queue(loop=loop)
 
     def get(self):
@@ -74,7 +74,7 @@ class TestWebSocketThread(unittest.TestCase):
 
     def test_upgrade(self):
         c = HttpClient()
-        handler = Echo()
+        handler = Echo(c._loop)
         ws = yield c.get(self.ws_echo, websocket_handler=handler)
         response = ws.handshake
         self.assertEqual(response.status_code, 101)
@@ -92,7 +92,7 @@ class TestWebSocketThread(unittest.TestCase):
 
     def test_ping(self):
         c = HttpClient()
-        handler = Echo()
+        handler = Echo(c._loop)
         ws = yield c.get(self.ws_echo, websocket_handler=handler)
         #
         # ASK THE SERVER TO SEND A PING FRAME
@@ -102,7 +102,7 @@ class TestWebSocketThread(unittest.TestCase):
 
     def test_pong(self):
         c = HttpClient()
-        handler = Echo()
+        handler = Echo(c._loop)
         ws = yield c.get(self.ws_echo, websocket_handler=handler)
         #
         ws.ping('TESTING CLIENT PING')
@@ -111,7 +111,7 @@ class TestWebSocketThread(unittest.TestCase):
 
     def test_close(self):
         c = HttpClient()
-        handler = Echo()
+        handler = Echo(c._loop)
         ws = yield c.get(self.ws_echo, websocket_handler=handler)
         self.assertEqual(ws.event('post_request').fired(), 0)
         ws.write('send close 1001')
@@ -136,6 +136,21 @@ class TestWebSocketThread(unittest.TestCase):
         self.assertEqual(ws.close_reason[0], 1001)
         self.assertTrue(ws._connection.closed)
 
+    def test_home(self):
+        c = HttpClient()
+        response = yield c.get(self.uri)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.headers['content-type'],
+                         'text/html; charset=utf-8')
+
+    def test_graph(self):
+        c = HttpClient()
+        handler = Echo(c._loop)
+        ws = yield c.get(self.ws_uri, websocket_handler=handler)
+        self.assertEqual(ws.event('post_request').fired(), 0)
+        message = yield handler.get()
+        self.assertTrue(message)
+
 
 @dont_run_with_thread
 class TestWebSocketProcess(TestWebSocketThread):
