@@ -77,19 +77,18 @@ And the backend will be selected via::
 
 .. _redis: http://redis.io/
 '''
-import sys
 import time
 from functools import partial
 from datetime import datetime, timedelta
 from hashlib import sha1
 
-from pulsar import (task, async, EventHandler, PulsarException, get_logger,
-                    Future, coroutine_return, get_request_loop)
-from pulsar.utils.pep import itervalues, to_string
+from pulsar import (task, async, EventHandler, PulsarException, yield_from,
+                    Future, coroutine_return, future_timeout,
+                    ConnectionRefusedError, CANCELLED_ERRORS)
+from pulsar.utils.pep import itervalues
 from pulsar.utils.security import gen_unique_id
-from pulsar.apps.data import create_store, PubSubClient, odm
-from pulsar.utils.log import (LocalMixin, lazyproperty, lazymethod,
-                              LazyString)
+from pulsar.apps.data import odm
+from pulsar.utils.log import lazyproperty, LazyString
 
 from .models import JobRegistry
 from . import states
@@ -153,11 +152,6 @@ class TaskConsumer(object):
     Instances of this consumer are created by the :class:`TaskBackend` when
     a task is executed.
 
-    .. attribute:: _loop
-
-        the :ref:`queue-based loop <queue-based-loop>` of the thread
-        executing the task.
-
     .. attribute:: task_id
 
         the :attr:`Task.id` being consumed.
@@ -176,8 +170,7 @@ class TaskConsumer(object):
         tasks from within a :ref:`job callable <job-callable>`.
     '''
     def __init__(self, backend, worker, task_id, job):
-        self._loop = get_request_loop()
-        self.logger = self._loop.logger
+        self.logger = worker.logger
         self.backend = backend
         self.worker = worker
         self.job = job
@@ -206,8 +199,11 @@ class Task(odm.Model):
     '''flag indicating the :ref:`task status <task-state>`
     '''
     kwargs = odm.PickleField()
+    '''Key-valued parameters used by this task
+    '''
     result = odm.PickleField()
-
+    '''Result as a json object
+    '''
     def done(self):
         '''Return ``True`` if the :class:`Task` has finshed.
 
@@ -285,12 +281,12 @@ class TaskBackend(EventHandler):
     def __init__(self, store, logger=None, task_paths=None,
                  schedule_periodic=False, backlog=1, max_tasks=0, name=None,
                  poll_timeout=None):
-        self.store = store
-        self._logger = logger
-        super(TaskBackend, self).__init__(self._loop,
+        super(TaskBackend, self).__init__(store._loop,
                                           many_times_events=('task_queued',
                                                              'task_started',
                                                              'task_done'))
+        self.store = store
+        self._logger = logger
         self.name = name
         self.task_paths = task_paths
         self.backlog = backlog
@@ -312,11 +308,6 @@ class TaskBackend(EventHandler):
             return 'task consumer %s' % self.store.dns
     __str__ = __repr__
 
-    @property
-    def _loop(self):
-        '''Eventloop running this task backend'''
-        return self.store._loop
-
     @property
     def num_concurrent_tasks(self):
         '''The number of :attr:`concurrent_tasks`.
@@ -406,16 +397,17 @@ class TaskBackend(EventHandler):
                 if task.done():  # task done, simply return it
                     done = callbacks.pop(task_id, None)
                     if done:
-                        done.set_result(task)
+                        done.set_result(task_id)
                 else:
                     done = callbacks.get(task_id)
                     if not done:
                         # No future, create one
                         callbacks[task_id] = done = Future(loop=self._loop)
-                    task = yield done
+                    yield done
+                    task = yield self.get_task(task_id)
                 coroutine_return(task)
 
-        fut = async(_(task_id), self._loop)
+        fut = async(yield_from(_(task_id)), loop=self._loop)
         return future_timeout(fut, timeout) if timeout else fut
 
     def get_tasks(self, ids):
@@ -429,7 +421,7 @@ class TaskBackend(EventHandler):
         # pubsub channels names from event names
         channels = tuple((self.channel(name) for name in self.events))
         pubsub.subscribe(*channels)
-        self.bind_event('task_done', self.task_done_callback)
+        self.bind_event('task_done', self._task_done_callback)
         return pubsub
 
     # #######################################################################
@@ -470,21 +462,20 @@ class TaskBackend(EventHandler):
     def start(self, worker):
         '''Invoked by the task queue ``worker`` when it starts.
         '''
-        assert self.task_poller is None
-        self.task_poller = worker._loop.call_soon(self.may_pool_task, worker)
+        self._may_pool_task(worker)
         self.logger.debug('started polling tasks')
-        store = self.store
 
     def close(self):
         '''Close this :class:`TaskBackend`.
 
         Invoked by the :class:`.Actor` when stopping.
         '''
-        if self.task_poller:
-            self.task_poller.cancel()
+        task = self.task_poller
+        if task:
+            task.cancel()
             self.task_poller = None
-            self.logger.debug('stopped polling tasks')
         self._pubsub.close()
+        return task
 
     def generate_task_ids(self, job, kwargs):
         '''An internal method to generate task unique identifiers.
@@ -599,21 +590,39 @@ class TaskBackend(EventHandler):
                         worker._loop.stop()
                         coroutine_return()
                 else:
-                    task = yield self.get_task()
+                    try:
+                        task = yield self.get_task()
+                    except ConnectionRefusedError:
+                        if worker.is_running():
+                            raise
+                        else:
+                            coroutine_return()
+                    except CANCELLED_ERRORS:
+                        self.logger.debug('stopped polling tasks')
+                        raise
                     if task:    # Got a new task
                         self.processed += 1
                         self.concurrent_tasks.add(task['id'])
-                        executor.submit(self._execute_task, worker, task)
+                        coro = self._execute_task(worker, task)
+                        executor.submit(yield_from, coro)
             else:
                 self.logger.debug('%s concurrent requests. Cannot poll.',
                                   self.num_concurrent_tasks)
                 next_time = 1
-        worker._loop.call_later(next_time, self.may_pool_task, worker)
+        self.task_poller = None
+        worker._loop.call_later(next_time, self._may_pool_task, worker)
+
+    def _may_pool_task(self, worker):
+        assert self.task_poller is None
+        if worker.is_running():
+            self.task_poller = self.may_pool_task(worker)
+        else:
+            worker._loop.call_soon(self._may_pool_task, worker)
 
     def _execute_task(self, worker, task):
         # Asynchronous execution of a Task. This method is called
         # on a separate thread of execution from the worker event loop thread.
-        logger = get_logger(worker.logger)
+        logger = worker.logger
         pubsub = self._pubsub
         task_id = task.id
         lock_id = task.get('lock_id')
@@ -643,7 +652,7 @@ class TaskBackend(EventHandler):
                 self.concurrent_tasks.discard(task_id)
                 coroutine_return(task_id)
         except TaskTimeout:
-            logger.info('%s timed-out', task_info)
+            logger.warning('%s timed-out', task_info)
             result = None
             status = states.REVOKED
         except Exception as exc:
@@ -667,22 +676,24 @@ class TaskBackend(EventHandler):
         entries = {}
         if not self.schedule_periodic:
             return entries
-        for name, task in self.registry.filter_types('periodic'):
-            every = task.run_every
+        for name, t in self.registry.filter_types('periodic'):
+            every = t.run_every
             if isinstance(every, int):
                 every = timedelta(seconds=every)
             if not isinstance(every, timedelta):
                 raise ValueError('Schedule %s is not a timedelta' % every)
-            entries[name] = SchedulerEntry(name, every, task.anchor)
+            entries[name] = SchedulerEntry(name, every, t.anchor)
         return entries
 
-    def task_done_callback(self, task_id, exc=None):
+    def _task_done_callback(self, task_id, exc=None):
         # Got a task_id from the ``<name>_task_done`` channel.
         # Check if a ``callback`` is available in the :attr:`callbacks`
         # dictionary. If so fire the callback with the ``task`` instance
-        # corresponsding to the input ``task_id``.
+        # corresponding to the input ``task_id``.
         # If a callback is not available, it must have been fired already
-        self.wait_for_task(task_id)
+        done = self.callbacks.pop(task_id, None)
+        if done:
+            done.set_result(task_id)
 
     def __call__(self, channel, message):
         # PubSub callback
@@ -778,6 +789,7 @@ class PulsarTaskBackend(TaskBackend):
     def store_client(self):
         return self.store.client()
 
+    @task
     def maybe_queue_task(self, task):
         free = True
         store = self.store
@@ -794,6 +806,7 @@ class PulsarTaskBackend(TaskBackend):
         else:
             coroutine_return()
 
+    @task
     def get_task(self, task_id=None):
         store = self.store
         if not task_id:
