@@ -168,21 +168,34 @@ sequential
 By default, test functions within a :class:`~unittest.TestCase`
 are run in asynchronous fashion. This means that several test functions
 may be executed at once depending on their return values.
+
 By specifying the :ref:`--sequential <setting-sequential>` command line option,
-the :class:`TestSuite`
-forces tests to be run in a sequential model, one after the other::
+the :class:`.TestSuite` forces test functions from a given
+:class:`~unittest.TestCase` to be run in a sequential way,
+one after the other::
 
     python runtests.py --sequential
 
-Alternatively, if you need to specify a Testcase which always runs its test
-functions in a sequential way, you can use the :func:`.sequential` decorator::
+Alternatively, if you need to specify a :class:`~unittest.TestCase` which
+always runs its test functions in a sequential way, you can use
+the :func:`.sequential` decorator::
 
-    from pulsar.apps.test import unittest, sequential
+    from pulsar.apps.test import sequential
 
     @sequential
     class MyTestCase(unittest.TestCase):
         ...
 
+Using the ``sequential`` option, does not mean only one test function
+is executed by the :class:`.TestSuite` at a given time. Indeed, several
+:class:`~unittest.TestCase` are executed at the same time and therefore
+each one of the may have one test function running.
+
+In order to run only one function at any time, the ``sequential``
+option should be used in conjunction with
+:ref:`--concurrent-tasks <setting-concurrent_tasks>` option::
+
+    python runtests.py --sequential --concurrent-tasks 1
 
 list labels
 ~~~~~~~~~~~~~~~~~~~
@@ -306,7 +319,7 @@ import unittest
 from functools import partial
 
 import pulsar
-from pulsar import multi_async
+from pulsar import multi_async, task
 from pulsar.apps import tasks
 from pulsar.apps.data import create_store
 from pulsar.apps.ds import PulsarDS
@@ -521,6 +534,7 @@ class TestSuite(tasks.TaskQueue):
                 sys.exit(code)
             return False
 
+    @task
     def monitor_start(self, monitor):
         '''When the monitor starts load all test classes into the queue'''
         # Create a datastore for this test suite
@@ -574,7 +588,7 @@ class TestSuite(tasks.TaskQueue):
                 raise ExitTest('Could not find any tests.')
         except ExitTest as e:   # pragma    nocover
             monitor.stream.writeln(str(e))
-            monitor.arbiter.stop()
+            monitor._loop.stop()
         except Exception:   # pragma    nocover
             monitor.logger.critical('Error occurred while starting tests',
                                     exc_info=True)
@@ -594,6 +608,7 @@ class TestSuite(tasks.TaskQueue):
         params['concurrency'] = self.cfg.concurrency
         return params
 
+    @task
     def _test_done(self, monitor, task_id=None, exc=None):
         runner = self.runner
         if task_id:
