@@ -148,33 +148,44 @@ from collections import Mapping, OrderedDict
 from functools import partial
 from inspect import isgenerator
 
-from pulsar import multi_async, Future, async, coroutine_return, chain_future
-from pulsar.utils.pep import iteritems, is_string, to_string, ispy3k
-from pulsar.utils.html import (slugify, INLINE_TAGS, escape,
-                               dump_data_value, child_tag)
+from pulsar import HttpException
+from pulsar import multi_async, async, coroutine_return, chain_future
+from pulsar.utils.pep import iteritems, to_string, ispy3k
+from pulsar.utils.slugify import slugify
+from pulsar.utils.html import INLINE_TAGS, escape, dump_data_value, child_tag
 from pulsar.utils.httpurl import remove_double_slash
 from pulsar.utils.system import json
 
-from .html import html_visitor
+from .html import html_visitor, newline
 
 __all__ = ['AsyncString', 'Html',
            'Json', 'HtmlDocument', 'Links', 'Scripts', 'Media',
            'html_factory', 'media_libraries']
 
 
-JS_DIR = os.path.join(os.path.dirname(__file__), 'js')
+_libs_url = 'http://quantmind.github.io/jslibs/libs.json'
+_media_libraries = None
 
 
-def load_pkg(name, dir=None):
-    p = os.path
-    dir = dir or JS_DIR
-    with open(os.path.join(dir, name)) as f:
-        data = f.read()
-    return json.loads(data)
-
-
-media_libraries = load_pkg('libs.json')
-javascript_dependencies = load_pkg('deps.json')
+def media_libraries():
+    global _media_libraries
+    if _media_libraries is None:
+        if os.path.isfile('libs.json'):     # pragma    nocover
+            with open('libs.json') as f:
+                data = f.read()
+            _media_libraries = json.loads(data)
+        else:
+            from pulsar import new_event_loop
+            from pulsar.apps.http import HttpClient
+            http = HttpClient(loop=new_event_loop())
+            try:
+                response = http.get(_libs_url)
+                _media_libraries = response.json()
+            except Exception:   # pragma    nocover
+                http.logger.error('Could not import media library',
+                                  exc_info=True)
+                _media_libraries = {'libs': {}, 'deps': {}}
+    return _media_libraries
 
 
 if ispy3k:
@@ -204,7 +215,6 @@ else:  # pragma nocover
 
 def stream_mapping(value, request=None):
     result = {}
-    async = False
     for key, value in iteritems(value):
         if isinstance(value, AsyncString):
             value = value.render(request)
@@ -391,19 +401,24 @@ class AsyncString(object):
         This method asynchronously wait for :meth:`stream` and subsequently
         returns a :class:`.WsgiResponse`.
         '''
-        response = request.response
-        response.content_type = self.content_type
-        if stream:
-            stream = stream[0]
-        else:
-            stream = multi_async(self.stream(request))
-            if stream.done():
-                stream = stream.result()
+        content_types = request.content_types
+        if not content_types or self._content_type in content_types:
+            response = request.response
+            response.content_type = self._content_type
+            response.encoding = self.charset
+            if stream:
+                stream = stream[0]
             else:
-                return chain_future(
-                    stream, callback=partial(self.http_response, request))
-        response.content = self.to_string(stream)
-        return response
+                stream = multi_async(self.stream(request))
+                if stream.done():
+                    stream = stream.result()
+                else:
+                    return chain_future(
+                        stream, callback=partial(self.http_response, request))
+            response.content = self.to_string(stream)
+            return response
+        else:
+            raise HttpException(status=415, msg=request.content_types)
 
     def to_string(self, streams):
         '''Called to transform the collection of
@@ -623,17 +638,6 @@ class Html(AsyncString):
         else:
             return result
 
-    def addDir(self, key, value=None):
-        '''Add a directive to the element
-
-        The ``value`` can be ``None``.
-        '''
-        attr = self._attr
-        if attr is None:
-            self._extra['attr'] = attr = {}
-        attr[key] = value or ''
-        return self
-
     def addClass(self, cn):
         '''Add the specific class names to the class set and return ``self``.
         '''
@@ -648,7 +652,7 @@ class Html(AsyncString):
                     self._extra['classes'] = classes = set()
                 add = classes.add
                 for cn in cn.split():
-                    add(slugify(cn, rtx='-'))
+                    add(slugify(cn))
         return self
 
     def hasClass(self, cn):
@@ -690,8 +694,11 @@ class Html(AsyncString):
 
     def css(self, mapping=None):
         '''Update the css dictionary if ``mapping`` is a dictionary, otherwise
- return the css value at ``mapping``. If ``mapping`` is not given, return the
- whole ``css`` dictionary if available.'''
+        return the css value at ``mapping``.
+
+        If ``mapping`` is not given, return the whole ``css`` dictionary
+        if available.
+        '''
         css = self._css
         if mapping is None:
             return css
@@ -726,11 +733,16 @@ class Html(AsyncString):
 
     def do_stream(self, request):
         self.add_media(request)
-        if self._tag and self._tag in INLINE_TAGS:
-            yield '<%s%s>' % (self._tag, self.flatatt())
+        tag = self._tag
+        n = '\n' if tag in newline else ''
+        if tag and tag in INLINE_TAGS:
+            yield '<%s%s>%s' % (tag, self.flatatt(), n)
         else:
-            if self._tag:
-                yield '<%s%s>' % (self._tag, self.flatatt())
+            if tag:
+                if not self._children:
+                    yield '<%s%s></%s>%s' % (tag, self.flatatt(), tag, n)
+                else:
+                    yield '<%s%s>%s' % (tag, self.flatatt(), n)
             if self._children:
                 for child in self._children:
                     if isinstance(child, AsyncString):
@@ -740,8 +752,8 @@ class Html(AsyncString):
                         yield async(child, getattr(request, '_loop', None))
                     else:
                         yield child
-            if self._tag:
-                yield '</%s>' % self._tag
+                if tag:
+                    yield '</%s>%s' % (tag, n)
 
     def _attrdata(self, cont, name, *val):
         if not name:
@@ -789,12 +801,17 @@ class Media(AsyncString):
     '''
     mediatype = None
 
-    def __init__(self, media_path, minified=False, known_libraries=None):
+    def __init__(self, media_path, minified=False, known_libraries=None,
+                 dependencies=None):
         super(Media, self).__init__()
         self.media_path = media_path
         self.minified = minified
-        self.known_libraries = (media_libraries if known_libraries is None
-                                else known_libraries)
+        if known_libraries is None:
+            known_libraries = media_libraries()['libs']
+        if dependencies is None:
+            dependencies = media_libraries()['deps']
+        self.known_libraries = known_libraries
+        self.dependencies = dependencies
 
     def is_relative(self, path):
         '''Check if ``path`` is a local relative path.
@@ -818,15 +835,21 @@ class Media(AsyncString):
 
         :return: A url path to insert in a HTML ``link`` or ``script``.
         '''
-        urlparams = ''
         ending = '.%s' % self.mediatype
+        minify = True
+        urlparams = ''
+        if isinstance(path, dict):
+            urlparams = path.get('urlparams', urlparams)
+            minify = path.get('minify', minify)
+            path = path['url']
         if path in self.known_libraries:
             lib = self.known_libraries[path]
             if isinstance(lib, dict):
                 urlparams = lib.get('urlparams', '')
+                minify = lib.get('minify', minify)
                 lib = lib['url']
             path = '%s%s' % (lib, ending)
-        if self.minified:
+        if self.minified and minify:
             if path.endswith(ending):
                 path = self._minify(path, ending)
         if not with_media_ending:
@@ -877,8 +900,8 @@ class Links(Media):
                 value.attr('media', media)
             if condition:
                 value = Html(None, '<!--[if %s]>\n' % condition,
-                             value, '\n<![endif]-->')
-            value = '%s\n' % value.render()
+                             value, '<![endif]-->\n')
+            value = value.render()
             if value not in self.children:
                 self.children.append(value)
 
@@ -891,8 +914,6 @@ class Scripts(Media):
     mediatype = 'js'
 
     def __init__(self, *args, **kwargs):
-        deps = kwargs.pop('dependencies', None)
-        self.dependencies = javascript_dependencies if deps is None else deps
         self.require_callback = kwargs.pop('require_callback', None)
         self.wait = kwargs.pop('wait', 200)
         self.required = []
@@ -902,8 +923,7 @@ class Scripts(Media):
     def script(self, src, type=None, **kwargs):
         type = type or 'application/javascript'
         path = self.absolute_path(src)
-        return '%s\n' % Html('script', src=path, type=type,
-                             **kwargs).render()
+        return Html('script', src=path, type=type, **kwargs).render()
 
     def require(self, *scripts):
         '''Add a ``script`` to the list of :attr:`required` scripts.
@@ -919,8 +939,10 @@ class Scripts(Media):
         for script in scripts:
             if script == 'require':
                 continue
-            script = script.strip()
-            if script not in self.known_libraries:
+            name = script
+            if isinstance(script, dict):
+                name = script['url']
+            if name.strip() not in self.known_libraries:
                 script = self.absolute_path(script)
             if script not in required:
                 required.append(script)
@@ -1036,13 +1058,11 @@ class Head(Html):
                  known_libraries=None, scripts_dependencies=None,
                  require_callback=None, **params):
         super(Head, self).__init__('head', **params)
-        if known_libraries is None:
-            known_libraries = media_libraries
-            scripts_dependencies = javascript_dependencies
         self.title = title
         self.append(Html(None, meta))
         self.append(Links(media_path, minified=minified,
-                          known_libraries=known_libraries))
+                          known_libraries=known_libraries,
+                          dependencies=scripts_dependencies))
         self.append(Embedded('style', type='text/css'))
         self.append(Embedded('script', type='text/javascript'))
         self.append(Scripts(media_path, minified=minified,
@@ -1080,7 +1100,7 @@ class Head(Html):
     def __get_js(self):
         return self._children[3]
 
-    def __set_js(self, css):
+    def __set_js(self, js):
         self._children[3] = js
     embedded_js = property(__get_js, __set_js)
 
@@ -1093,7 +1113,7 @@ class Head(Html):
 
     def do_stream(self, request):
         if self.title:
-            self._children.insert(0, '<title>%s</title>' % self.title)
+            self._children.insert(0, '<title>%s</title>\n' % self.title)
         return super(Head, self).do_stream(request)
 
     def add_meta(self, **kwargs):
@@ -1152,8 +1172,8 @@ class HtmlDocument(Html):
     '''
     _template = ('<!DOCTYPE html>\n'
                  '<html%s>\n'
-                 '%s\n%s'
-                 '\n</html>')
+                 '%s%s'
+                 '</html>')
 
     def __init__(self, title=None, media_path='/media/', charset=None,
                  minified=False, known_libraries=None, require_callback=None,
