@@ -2,7 +2,7 @@
 
 .. note::
 
-    The :mod:`pulsar.apps.wsgi.route` module was originally from the routing
+    The :mod:`~pulsar.apps.wsgi.route` module was originally from the routing
     module in werkzeug_. Original License:
 
     copyright (c) 2011 by the Werkzeug Team. License BSD
@@ -26,7 +26,6 @@ Paths::
 
     # accept any path (including slashes)
     Route('<path:pages>')
-    # accept an integer between 1 and 200 only
     Route('<path:pages>/edit')
 
 
@@ -57,6 +56,7 @@ from pulsar import Http404
 from pulsar.utils.httpurl import (iteritems, iri_to_uri, remove_double_slash,
                                   ENCODE_URL_METHODS, ENCODE_BODY_METHODS)
 from pulsar.utils.pep import to_string
+from pulsar.utils.slugify import slugify
 
 
 __all__ = ['route', 'Route']
@@ -134,25 +134,48 @@ class route(object):
 
     Typical usage::
 
-        from pulsar.apps import wsgi
+        from pulsar.apps.wsgi import Router, route
 
-        class View(wsgi.Router):
+        class View(Router):
 
-            wsgi.route('/foo')
-            def handle1(self, request):
+            def get(self, request):
                 ...
 
-            wsgi.route('/bla', method='post')
+            @route()
+            def foo(self, request):
+                ...
+
+            @route('/bla', method='post')
             def handle2(self, request):
                 ...
 
 
-    In this example, ``View`` is the **parent router**.
+    In this example, ``View`` is the **parent router** which handle get
+    requests only.
 
     The decorator injects the :attr:`rule_method` attribute to the
-    method it decorates. The attribute is a four elements tuple
-    contains the :class:`Route`, the HTTP ``method``, a
-    dictionary of additional ``parameters`` and the ``position`` for ordering.
+    method it decorates. The attribute is a **five elements tuple**
+    which contains the :class:`Route`, the HTTP ``method``, a
+    dictionary of additional ``parameters``, ``position`` and
+    the ``order``. Position and order are internal integers used by the
+    :class:`.Router` when deciding the order of url matching.
+    If ``position`` is not passed,
+    the order will be given by the position of each method in the
+    :class:`.Router` class.
+
+    The decorated method are stored in the :attr:`.Router.rule_methods` class
+    attribute.
+
+        >>> len(View.rule_methods)
+        2
+        >>> View.rule_methods['foo'].rule
+        /foo
+        >>> View.rule_methods['foo'].method
+        'get'
+        >>> View.rule_methods['handle2'].rule
+        /bla
+        >>> View.rule_methods['handle2'].method
+        'post'
 
     Check the :ref:`HttpBin example <tutorials-httpbin>`
     for a sample usage.
@@ -165,8 +188,7 @@ class route(object):
     :param position: Optional positioning of the router within the
         list of child routers of the parent router
     :param parameters: Additional parameters used when initialising
-        the :class:`pulsar.apps.wsgi.handlers.Router` created by this
-        decorator
+        the :class:`.Router` created by this decorator
 
     '''
     creation_count = 0
@@ -193,8 +215,10 @@ class route(object):
             if m in ENCODE_URL_METHODS or m in ENCODE_BODY_METHODS:
                 method = m
                 bits = bits[1:]
+        name = self.parameters.get('name', '_'.join(bits))
+        self.parameters['name'] = name
         method = (self.method or method or 'get').lower()
-        rule = Route(self.rule or '_'.join(bits), defaults=self.defaults)
+        rule = Route(self.rule or name, defaults=self.defaults)
         callable.rule_method = rule_info(rule, method, self.parameters,
                                          self.position, self.order)
         return callable
@@ -286,10 +310,7 @@ class Route(object):
 
         Derived from :attr:`rule` replacing underscores and dashes.
         '''
-        name = self.rule.replace('/', ' ')
-        if not self.variables:
-            name = name.replace('_', ' ').replace('-', ' ')
-        return name.strip()
+        return slugify(self.rule, separator='_')
 
     @property
     def regex(self):
@@ -302,7 +323,10 @@ class Route(object):
     def bits(self):
         return tuple((b[1] for b in self.breadcrumbs))
 
+    @property
     def ordered_variables(self):
+        '''Tuple of ordered url :attr:`variables`
+        '''
         return tuple((b for dyn, b in self.breadcrumbs if dyn))
 
     def __hash__(self):
@@ -374,8 +398,8 @@ class Route(object):
 
     def split(self):
         '''Return a two element tuple containing the parent route and
-the last url bit as route. If this route is the root route, it returns
-the root route and ``None``. '''
+        the last url bit as route. If this route is the root route, it returns
+        the root route and ``None``. '''
         rule = self.rule
         if not self.is_leaf:
             rule = rule[:-1]
