@@ -70,13 +70,13 @@ from email.utils import parsedate_tz, mktime_tz
 
 from pulsar.utils.httpurl import http_date, CacheControl
 from pulsar.utils.structures import AttributeDictionary, OrderedDict
+from pulsar.utils.slugify import slugify
 from pulsar import Http404, HttpException
 
 from .route import Route
 from .utils import wsgi_request
 from .content import Html
 from .structures import ContentAccept
-from .wrappers import redirect
 
 
 __all__ = ['Router', 'MediaRouter', 'FileRouter', 'MediaMixin',
@@ -174,6 +174,13 @@ class Router(RouterType('RouterBase', (object,), {})):
         exception of :attr:`response_content_types` and
         :attr:`response_wrapper`
 
+    .. attribute:: rule_methods
+
+        A class attribute built during class creation. It is an ordered
+        dictionary mapping method names with a five-elements tuple
+        containing information
+        about a child route (See the :class:`.route` decorator).
+
     .. attribute:: routes
 
         List of children :class:`Router` of this :class:`Router`.
@@ -184,7 +191,7 @@ class Router(RouterType('RouterBase', (object,), {})):
 
     .. attribute:: response_content_types
 
-        a list/tuple of possible content types of a response to a
+        A list/tuple of possible content types of a response to a
         client request.
 
         The client request must accept at least one of the response content
@@ -200,28 +207,19 @@ class Router(RouterType('RouterBase', (object,), {})):
                 ...
                 return handler(request)
 
-    .. attribute:: allows_redirects
-
-        boolean indicating if this router can redirect requests to valid urls
-        within this router and its children. For example, if a router serves
-        the '/echo' url but not the ``/echo/`` one, a request on ``/echo/``
-        will be redirected to ``/echo``.
-
-        Default: ``False``
-
     .. attribute:: parameters
 
         A :class:`.AttributeDictionary` of parameters for
         this :class:`Router`. Parameters are created at initialisation from
         the ``parameters`` class attribute and the key-valued parameters
-        passed to the ``__init__`` method for which the value is not callable.
+        passed to the ``__init__`` method which are available in the
+        class ``parameters`` attribute.
     '''
     _creation_count = 0
     _parent = None
     _name = None
 
     response_content_types = RouterParam(None)
-    allows_redirects = RouterParam(False)
     response_wrapper = RouterParam(None)
 
     def __init__(self, rule, *routes, **parameters):
@@ -232,7 +230,7 @@ class Router(RouterType('RouterBase', (object,), {})):
         self._route = rule
         self._name = parameters.pop('name', rule.name)
         self.routes = []
-        # add routes specified via the initialiser
+        # add routes specified via the initialiser first
         for router in routes:
             self.add_child(router)
         # copy parameters
@@ -241,13 +239,13 @@ class Router(RouterType('RouterBase', (object,), {})):
             rule, method, params, _, _ = rule_method
             rparameters = params.copy()
             handler = getattr(self, name)
-            router = self.add_child(Router(rule, **rparameters))
+            router = self.add_child(self.make_router(rule, **rparameters))
             setattr(router, method, handler)
         for name, value in parameters.items():
             if name in self.parameters:
                 self.parameters[name] = value
             else:
-                setattr(self, name, value)
+                setattr(self, slugify(name, separator='_'), value)
 
     @property
     def route(self):
@@ -294,15 +292,6 @@ class Router(RouterType('RouterBase', (object,), {})):
     def parent(self):
         return self._parent
 
-    @property
-    def default_content_type(self):
-        '''The default content type for responses.
-
-        This is the first element in the :attr:`response_content_types` list.
-        '''
-        ct = self.response_content_types
-        return ct[0] if ct else None
-
     @property
     def creation_count(self):
         '''Integer for sorting :class:`Router` by creation.
@@ -327,6 +316,14 @@ class Router(RouterType('RouterBase', (object,), {})):
         '''
         return self.full_route.url(**urlargs)
 
+    def getparam(self, name, default=None):
+        '''A parameter in this :class:`.Router`
+        '''
+        try:
+            return getattr(self, name)
+        except AttributeError:
+            return default
+
     def __getattr__(self, name):
         '''Check the value of a :attr:`parameters` ``name``.
 
@@ -334,42 +331,23 @@ class Router(RouterType('RouterBase', (object,), {})):
         :attr:`parent` :class:`Router` if it exists.
         '''
         if not name.startswith('_'):
-            return self.get_parameter(name, False)
-        self.no_param(name)
-
-    def get_parameter(self, name, safe=True):
-        value = self.parameters.get(name)
-        if value is None:
-            if self._parent:
-                return self._parent.get_parameter(name, safe)
-            elif name in self.parameters:
-                return value
-            elif not safe:
-                self.no_param(name)
-        else:
-            return value
-
-    def no_param(self, name):
-        raise AttributeError("'%s' object has no attribute '%s'" %
-                             (self.__class__.__name__, name))
+            return self._get_router_parameter(name, False)
+        self._no_param(name)
 
     def content_type(self, request):
         '''Evaluate the content type for the response to a client ``request``.
 
         The method uses the :attr:`response_content_types` parameter of
         accepted content types and the content types accepted by the client
-        and figure out the best match.
+        ``request`` and figures out the best match.
         '''
-        return request.content_types.best_match(self.response_content_types)
-
-    def accept_content_type(self, content_type):
-        '''Check if ``content_type`` is accepted by this :class:`Router`.
-
-        Return the best mach or ``None`` if not accepted.'''
-        response_content_types = self.response_content_types
-        if response_content_types:
-            return ContentAccept(
-                [(content_type, 1)]).best_match(response_content_types)
+        content_types = self.response_content_types
+        ct = request.content_types.best_match(content_types)
+        if ct and '*' in ct:
+            ct = None
+        if not ct and content_types:
+            raise HttpException(status=415, msg=request.content_types)
+        return ct
 
     def __repr__(self):
         return self.route.__repr__()
@@ -381,17 +359,6 @@ class Router(RouterType('RouterBase', (object,), {})):
         if router_args:
             router, args = router_args
             return router.response(environ, args)
-        elif self.allows_redirects:
-            if self.route.is_leaf:
-                if path.endswith('/'):
-                    router_args = self.resolve(path[:-1])
-                    if router_args is not None:
-                        return self.redirect('/%s' % path[:-1])
-            else:
-                if not path.endswith('/'):
-                    router_args = self.resolve('%s/' % path)
-                    if router_args is not None:
-                        return self.redirect('/%s/' % path)
 
     def resolve(self, path, urlargs=None):
         '''Resolve a path and return a ``(handler, urlargs)`` tuple or
@@ -419,37 +386,32 @@ class Router(RouterType('RouterBase', (object,), {})):
         this method to produce the WSGI response.
         '''
         request = wsgi_request(environ, self, args)
-        # Set the response content type
         request.response.content_type = self.content_type(request)
         method = request.method.lower()
         callable = getattr(self, method, None)
         if callable is None:
-            raise HttpException(status=405,
-                                msg='Method "%s" not allowed' % method)
+            raise HttpException(status=405)
         response_wrapper = self.response_wrapper
         if response_wrapper:
             return response_wrapper(callable, request)
         return callable(request)
 
-    def redirect(self, path, **kw):
-        '''Redirect to a different ``path``
-        '''
-        return redirect(path, **kw)
-
     def add_child(self, router):
         '''Add a new :class:`Router` to the :attr:`routes` list.
         '''
         assert isinstance(router, Router), 'Not a valid Router'
         assert router is not self, 'cannot add self to children'
+        #
+        # Remove from previous parent
+        if router.parent:
+            router.parent.remove_child(router)
+        router._parent = self
         # Loop over available routers to check it the router
         # is already available
         for r in self.routes:
             if r.route == router.route:
                 r.parameters.update(router.parameters)
                 return r
-        if router.parent:
-            router.parent.remove_child(router)
-        router._parent = self
         self.routes.append(router)
         return router
 
@@ -460,10 +422,17 @@ class Router(RouterType('RouterBase', (object,), {})):
             router._parent = None
 
     def get_route(self, name):
-        '''Get a child :class:`Router` by its :attr:`name`.'''
+        '''Get a child :class:`Router` by its :attr:`name`.
+
+        This method search child routes recursively.
+        '''
         for route in self.routes:
             if route.name == name:
                 return route
+        for child in self.routes:
+            route = child.get_route(name)
+            if route:
+                return route
 
     def link(self, *args, **urlargs):
         '''Return an anchor :class:`Html` element with the `href` attribute
@@ -485,14 +454,33 @@ class Router(RouterType('RouterBase', (object,), {})):
             parent = parent._parent
         return parent is not None
 
+    def make_router(self, rule, cls=None, **params):
+        '''Create a new :class:`.Router` from a ``rule`` and parameters.
 
-class MediaMixin(Router):
-    response_content_types = RouterParam(('application/octet-stream',
-                                          'text/css',
-                                          'application/javascript',
-                                          'text/html'))
-    cache_control = CacheControl(maxage=86400)
-    _file_path = ''
+        This method is used during initialisation when building child
+        Routers from the :attr:`rule_methods`.
+        '''
+        cls = cls or Router
+        return cls(rule, **params)
+
+    def _no_param(self, name):
+        raise AttributeError("'%s' object has no attribute '%s'" %
+                             (self.__class__.__name__, name))
+
+    def _get_router_parameter(self, name, safe=True):
+        value = self.parameters.get(name)
+        if value is None:
+            if self._parent:
+                return self._parent._get_router_parameter(name, safe)
+            elif name in self.parameters:
+                return value
+            elif not safe:
+                self._no_param(name)
+        else:
+            return value
+
+
+class MediaMixin(object):
 
     def serve_file(self, request, fullpath, status_code=None):
         # Respect the If-Modified-Since header.
@@ -501,7 +489,8 @@ class MediaMixin(Router):
         response = request.response
         if content_type:
             response.content_type = content_type
-        response.encoding = encoding
+        if encoding:
+            response.encoding = encoding
         if not (status_code or self.was_modified_since(
                 request.environ.get('HTTP_IF_MODIFIED_SINCE'),
                 statobj[stat.ST_MTIME],
@@ -551,9 +540,6 @@ class MediaMixin(Router):
         names.extend(files)
         return self.static_index(request, names)
 
-    def html_title(self, request):
-        return 'Index of %s' % request.path
-
     def static_index(self, request, links):
         doc = request.html_document
         doc.title = 'Index of %s' % request.path
@@ -563,7 +549,7 @@ class MediaMixin(Router):
         return doc.http_response(request)
 
 
-class MediaRouter(MediaMixin):
+class MediaRouter(Router, MediaMixin):
     '''A :class:`Router` for serving static media files from a given
     directory.
 
@@ -585,13 +571,16 @@ class MediaRouter(MediaMixin):
 
         The default file to serve when a directory is requested.
     '''
-    def __init__(self, rute, path, show_indexes=False, mapping=None,
+    response_content_types = RouterParam(('application/octet-stream',
+                                          'text/css',
+                                          'application/javascript',
+                                          'text/html'))
+    cache_control = CacheControl(maxage=86400)
+
+    def __init__(self, rule, path, show_indexes=False, mapping=None,
                  default_suffix=None, default_file='index.html',
                  raise_404=True):
-        rute = '%s/<path:path>' % rute
-        if rute.startswith('/'):
-            rute = rute[1:]
-        super(MediaRouter, self).__init__(rute)
+        super(MediaRouter, self).__init__('%s/<path:path>' % rule)
         self._mapping = mapping or {}
         self._default_suffix = default_suffix
         self._default_file = default_file
@@ -609,6 +598,8 @@ class MediaRouter(MediaMixin):
         if os.path.isdir(fullpath) and self._default_file:
             file = os.path.join(fullpath, self._default_file)
             if os.path.isfile(file):
+                if not request.path.endswith('/'):
+                    return request.redirect('%s/' % request.path)
                 fullpath = file
         #
         if os.path.isdir(fullpath):
@@ -627,8 +618,15 @@ class MediaRouter(MediaMixin):
             raise Http404
 
 
-class FileRouter(MediaMixin):
-    '''A Router for a single file.'''
+class FileRouter(Router, MediaMixin):
+    '''A Router for a single file
+    '''
+    response_content_types = RouterParam(('application/octet-stream',
+                                          'text/css',
+                                          'application/javascript',
+                                          'text/html'))
+    cache_control = CacheControl(maxage=86400)
+
     def __init__(self, route, file_path, status_code=None, raise_404=True):
         super(FileRouter, self).__init__(route)
         self._status_code = status_code
