@@ -19,8 +19,8 @@ import socket
 from wsgiref.handlers import format_date_time
 
 import pulsar
-from pulsar import (reraise, HttpException, ProtocolError, Future, in_loop,
-                    chain_future)
+from pulsar import (reraise, HttpException, ProtocolError, Future, task,
+                    From, chain_future)
 from pulsar.utils.pep import is_string, native_str
 from pulsar.utils.httpurl import (Headers, unquote, has_empty_content,
                                   host_and_port_default, http_parser,
@@ -36,6 +36,7 @@ __all__ = ['HttpServerResponse', 'MAX_CHUNK_SIZE', 'test_wsgi_environ']
 
 
 MAX_CHUNK_SIZE = 65536
+MAX_TIME_IN_LOOP = 0.5
 
 
 class FakeConnection(object):
@@ -289,8 +290,8 @@ class HttpServerResponse(ProtocolConsumer):
     SERVER_SOFTWARE = pulsar.SERVER_SOFTWARE
     ONE_TIME_EVENTS = ProtocolConsumer.ONE_TIME_EVENTS + ('on_headers',)
 
-    def __init__(self, wsgi_callable, cfg, server_software=None):
-        super(HttpServerResponse, self).__init__()
+    def __init__(self, wsgi_callable, cfg, server_software=None, loop=None):
+        super(HttpServerResponse, self).__init__(loop=loop)
         self.wsgi_callable = wsgi_callable
         self.cfg = cfg
         self.parser = http_parser(kind=0)
@@ -311,21 +312,23 @@ class HttpServerResponse(ProtocolConsumer):
             self._stream = StreamReader(headers, parser, self.transport)
             self._response(self.wsgi_environ())
         #
-        done = parser.is_message_complete()
-        if done and not self._stream.on_message_complete.done():
-            self._stream.on_message_complete.set_result(None)
-        #
-        if processed < len(data):
-            if not done:
-                # This is a parsing error, the client must have sent
-                # bogus data
-                raise ProtocolError
-            else:
+        if parser.is_message_complete():
+            #
+            # Stream has the whole body
+            if not self._stream.on_message_complete.done():
+                self._stream.on_message_complete.set_result(None)
+
+            if processed < len(data):
                 if not self._buffer:
                     self._buffer = data[processed:]
                     self.bind_event('post_request', self._new_request)
                 else:
                     self._buffer += data[processed:]
+        #
+        elif processed < len(data):
+            # This is a parsing error, the client must have sent
+            # bogus data
+            raise ProtocolError
 
     @property
     def status(self):
@@ -408,30 +411,33 @@ class HttpServerResponse(ProtocolConsumer):
         Required by the WSGI specification.
 
         :param data: bytes to write
-        :param force: Optional flag used internally.
+        :param force: Optional flag used internally
+        :return: a :class:`~asyncio.Future` or the number of bytes written
         '''
+        write = super(HttpServerResponse, self).write
+        chunks = []
         if not self._headers_sent:
             tosend = self.get_headers()
             self._headers_sent = tosend.flat(self.version, self.status)
             self.fire_event('on_headers')
-            self.transport.write(self._headers_sent)
+            chunks.append(self._headers_sent)
         if data:
             if self.chunked:
-                chunks = []
                 while len(data) >= MAX_CHUNK_SIZE:
                     chunk, data = data[:MAX_CHUNK_SIZE], data[MAX_CHUNK_SIZE:]
                     chunks.append(chunk_encoding(chunk))
                 if data:
                     chunks.append(chunk_encoding(data))
-                self.transport.write(b''.join(chunks))
             else:
-                self.transport.write(data)
+                chunks.append(data)
         elif force and self.chunked:
-            self.transport.write(chunk_encoding(data))
+            chunks.append(chunk_encoding(data))
+        if chunks:
+            return write(b''.join(chunks))
 
     ########################################################################
     #    INTERNALS
-    @in_loop
+    @task
     def _response(self, environ):
         exc_info = None
         response = None
@@ -447,23 +453,38 @@ class HttpServerResponse(ProtocolConsumer):
                     response = handle_wsgi_error(environ, exc_info)
                 #
                 if isinstance(response, Future):
-                    response = yield response
+                    response = yield From(response)
                 #
                 if exc_info:
                     self.start_response(response.status,
                                         response.get_headers(), exc_info)
                 #
+                # Do the actual writing
+                loop = self._loop
+                start = loop.time()
                 for chunk in response:
                     if isinstance(chunk, Future):
-                        chunk = yield chunk
-                    self.write(chunk)
+                        chunk = yield From(chunk)
+                        start = loop.time()
+                    result = self.write(chunk)
+                    if isinstance(result, Future):
+                        yield From(result)
+                        start = loop.time()
+                    else:
+                        time_in_loop = loop.time() - start
+                        if time_in_loop > MAX_TIME_IN_LOOP:
+                            self.logger.debug(
+                                'Released the event loop after %.3f seconds',
+                                time_in_loop)
+                            yield From(None)
+                            start = loop.time()
                 #
-                # make sure we write headers
+                # make sure we write headers and last chunk if needed
                 self.write(b'', True)
 
             except IOError:     # client disconnected, end this connection
                 self.finished()
-            except Exception as exc:
+            except Exception:
                 if wsgi_request(environ).cache.handle_wsgi_error:
                     self.keep_alive = False
                     self.connection.close()
@@ -535,9 +556,6 @@ class HttpServerResponse(ProtocolConsumer):
                              ('Date', format_date_time(time.time()))])
         return environ
 
-    def _new_request(self, response):
-        connection = response._connection
-        if not connection.closed:
-            connection.data_received(response._buffer)
-            return connection._current_consumer
-        return response
+    def _new_request(self, _, exc=None):
+        connection = self._connection
+        connection.data_received(self._buffer)
