@@ -1,12 +1,12 @@
 import sys
-from functools import partial
 
 import pulsar
 from pulsar.utils.internet import nice_address, format_address
 
-from .futures import multi_async, in_loop, task, coroutine_return
+from .futures import multi_async, task, Future
 from .events import EventHandler
-from .access import asyncio, get_event_loop, new_event_loop
+from .mixins import FlowControl, Timeout
+from .access import asyncio, get_io_loop, From, ConnectionResetError
 
 
 __all__ = ['ProtocolConsumer',
@@ -18,11 +18,8 @@ __all__ = ['ProtocolConsumer',
            'DatagramServer']
 
 
-BIG = 2**31
-
-
 class ProtocolConsumer(EventHandler):
-    '''The consumer of data for a server or client :class:`Connection`.
+    '''The consumer of data for a server or client :class:`.Connection`.
 
     It is responsible for receiving incoming data from an end point via the
     :meth:`Connection.data_received` method, decoding (parsing) and,
@@ -167,7 +164,11 @@ class ProtocolConsumer(EventHandler):
 
         By default it calls the :meth:`finished` method. It can be overwritten
         to handle the potential exception ``exc``.'''
-        return self.finished(exc=exc)
+        # TODO: decide how to handle connection_lost when no exception occurs
+        # Set the first positional parameter to None so that if the
+        # connection was dropped without exception it returns None
+        # rather than the protocol consumer
+        return self.finished(None)
 
     def finished(self, *arg, **kw):
         '''Fire the ``post_request`` event if it wasn't already fired.
@@ -175,6 +176,17 @@ class ProtocolConsumer(EventHandler):
         if not self.event('post_request').fired():
             return self.fire_event('post_request', *arg, **kw)
 
+    def write(self, data):
+        '''Delegate writing to the underlying :class:`.Connection`
+
+        Return an empty tuple or a :class:`~asyncio.Future`
+        '''
+        c = self._connection
+        if c:
+            return c.write(data)
+        else:
+            raise RuntimeError('No connection')
+
     def _data_received(self, data):
         # Called by Connection, it updates the counters and invoke
         # the high level data_received method which must be implemented
@@ -193,7 +205,7 @@ class ProtocolConsumer(EventHandler):
             c._current_consumer = None
 
 
-class PulsarProtocol(EventHandler):
+class PulsarProtocol(EventHandler, FlowControl):
     '''A mixin class for both :class:`.Protocol` and
     :class:`.DatagramProtocol`.
 
@@ -204,18 +216,18 @@ class PulsarProtocol(EventHandler):
     * ``connection_lost``
     '''
     ONE_TIME_EVENTS = ('connection_made', 'connection_lost')
+    MANY_TIMES_EVENTS = ('data_received', 'data_processed',
+                         'before_write', 'after_write')
 
     _transport = None
-    _idle_timeout = None
     _address = None
     _type = 'server'
 
-    def __init__(self, session=1, producer=None, timeout=0):
-        super(PulsarProtocol, self).__init__()
+    def __init__(self, loop=None, session=1, producer=None, **kw):
+        super(PulsarProtocol, self).__init__(loop)
+        FlowControl.__init__(self, **kw)
         self._session = session
-        self._timeout = timeout
         self._producer = producer
-        self.bind_event('connection_lost', self._cancel_timeout)
 
     def __repr__(self):
         address = self._address
@@ -257,20 +269,6 @@ class PulsarProtocol(EventHandler):
         '''
         return self._address
 
-    @property
-    def timeout(self):
-        '''Number of seconds to keep alive this connection when idle.
-
-        A value of ``0`` means no timeout.'''
-        return self._timeout
-
-    @property
-    def _loop(self):
-        '''The :attr:`transport` event loop.
-        '''
-        if self._transport:
-            return self._transport._loop
-
     @property
     def producer(self):
         '''The producer of this :class:`Protocol`.
@@ -296,8 +294,6 @@ class PulsarProtocol(EventHandler):
         '''Sets the :attr:`transport`, fire the ``connection_made`` event
         and adds a :attr:`timeout` for idle connections.
         '''
-        if self._transport is not None:
-            self._cancel_timeout()
         self._transport = transport
         addr = self._transport.get_extra_info('peername')
         if not addr:
@@ -306,58 +302,60 @@ class PulsarProtocol(EventHandler):
         self._address = addr
         # let everyone know we have a connection with endpoint
         self.fire_event('connection_made')
-        self._add_idle_timeout()
 
     def connection_lost(self, exc=None):
         '''Fires the ``connection_lost`` event.
         '''
-        self.fire_event('connection_lost', exc=exc)
+        self.fire_event('connection_lost')
 
     def eof_received(self):
         '''The socket was closed from the remote end'''
 
-    def set_timeout(self, timeout):
-        '''Set a new :attr:`timeout` for this connection.'''
-        self._cancel_timeout()
-        self._timeout = timeout
-        self._add_idle_timeout()
-
     def info(self):
-        connection = {'session': self._session,
-                      'timeout': self._timeout}
-        info = {'connection': connection}
+        info = {'connection': {'session': self._session}}
         if self._producer:
             info.update(self._producer.info())
         return info
 
-    ########################################################################
-    #    INTERNALS
-    def _timed_out(self):
-        self.close()
-        self.logger.debug('Closed idle %s.', self)
-
-    def _add_idle_timeout(self):
-        if not self.closed and not self._idle_timeout and self._timeout:
-            self._idle_timeout = self._loop.call_later(self._timeout,
-                                                       self._timed_out)
-
-    def _cancel_timeout(self, *args, **kw):
-        if self._idle_timeout:
-            self._idle_timeout.cancel()
-            self._idle_timeout = None
-
 
 class Protocol(PulsarProtocol, asyncio.Protocol):
     '''An :class:`asyncio.Protocol` with :ref:`events <event-handling>`
     '''
+    _data_received_count = 0
+
+    def write(self, data):
+        '''Write ``data`` into the wire.
+
+        Returns an empty tuple or a :class:`~asyncio.Future` if this
+        protocol has paused writing.
+        '''
+        t = self._transport
+        if t:
+            if t._closing:  # Uses private variable.
+                raise ConnectionResetError('Connection lost')
+            if self._paused:
+                # # Uses private variable once again!
+                # This occurs when the protocol is paused from writing
+                # but another data ready callback is fired in the same
+                # event-loop frame
+                self.logger.debug('protocol cannot write, add data to the '
+                                  'transport buffer')
+                t._buffer.extend(data)
+            else:
+                self.fire_event('before_write')
+                t.write(data)
+                self.fire_event('after_write')
+            return self._write_waiter or ()
+        else:
+            raise ConnectionResetError('No Transport')
 
 
 class DatagramProtocol(PulsarProtocol, asyncio.DatagramProtocol):
     '''An ``asyncio.DatagramProtocol`` with events`'''
 
 
-class Connection(Protocol):
-    '''A :class:`Protocol` to handle multiple TCP requests/responses.
+class Connection(Protocol, Timeout):
+    '''A :class:`.FlowControl` to handle multiple TCP requests/responses.
 
     It is a class which acts as bridge between a
     :ref:`transport <asyncio-transport>` and a :class:`.ProtocolConsumer`.
@@ -372,13 +370,14 @@ class Connection(Protocol):
 
         number of separate requests processed.
     '''
-    _current_consumer = None
-
-    def __init__(self, consumer_factory=None, **kw):
+    def __init__(self, consumer_factory=None, timeout=None,
+                 low_limit=None, high_limit=None, **kw):
         super(Connection, self).__init__(**kw)
-        self._processed = 0
-        self._consumer_factory = consumer_factory
         self.bind_event('connection_lost', self._connection_lost)
+        self._processed = 0
+        self._current_consumer = None
+        self._consumer_factory = consumer_factory
+        self.timeout = timeout
 
     def current_consumer(self):
         '''The :class:`ProtocolConsumer` currently handling incoming data.
@@ -395,7 +394,6 @@ class Connection(Protocol):
         assert self._current_consumer is None, 'Consumer is not None'
         self._current_consumer = consumer
         consumer._connection = self
-        consumer._loop = self._loop
         consumer.connection_made(self)
 
     def data_received(self, data):
@@ -404,11 +402,14 @@ class Connection(Protocol):
         Once done set a timeout for idle connections when a
         :attr:`~Protocol.timeout` is a positive number (of seconds).
         '''
-        self._cancel_timeout()
+        self._data_received_count = self._data_received_count + 1
+        self.fire_event('data_received', data=data)
         while data:
             consumer = self.current_consumer()
             data = consumer._data_received(data)
-        self._add_idle_timeout()
+            if isinstance(data, Future):
+                break
+        self.fire_event('data_received', data=data)
 
     def upgrade(self, consumer_factory):
         '''Upgrade the :func:`_consumer_factory` callable.
@@ -434,8 +435,10 @@ class Connection(Protocol):
 
     def info(self):
         info = super(Connection, self).info()
-        connection = info['connection']
-        connection.update({'request_processed': self._processed})
+        c = info['connection']
+        c['request_processed'] = self._processed
+        c['data_processed_count'] = self._data_received_count
+        c['timeout'] = self.timeout
         return info
 
     def _build_consumer(self, _, exc=None):
@@ -443,7 +446,7 @@ class Connection(Protocol):
             consumer = self._producer.build_consumer(self._consumer_factory)
             self.set_consumer(consumer)
 
-    def _connection_lost(self, conn, exc=None):
+    def _connection_lost(self, _, exc=None):
         '''It performs these actions in the following order:
 
         * Fires the ``connection_lost`` :ref:`one time event <one-time-event>`
@@ -452,17 +455,14 @@ class Connection(Protocol):
         * Invokes the :meth:`ProtocolConsumer.connection_lost` method in the
           :meth:`current_consumer`.
           '''
-        if conn._current_consumer:
-            conn._current_consumer.connection_lost(exc)
+        if self._current_consumer:
+            self._current_consumer.connection_lost(exc)
 
 
 class Producer(EventHandler):
     '''An Abstract :class:`.EventHandler` class for all producers of
-    connections.
+    socket (client and servers)
     '''
-    _requests_processed = 0
-    _sessions = 0
-
     protocol_factory = None
     '''A callable producing protocols.
 
@@ -471,9 +471,15 @@ class Producer(EventHandler):
         protocol_factory(session, producer, **params)
     '''
 
-    def __init__(self, loop):
-        self._loop = loop or get_event_loop() or new_event_loop()
-        super(Producer, self).__init__(self._loop)
+    def __init__(self, loop, protocol_factory=None, name=None,
+                 max_requests=None, logger=None):
+        super(Producer, self).__init__(get_io_loop(loop))
+        self.protocol_factory = protocol_factory or self.protocol_factory
+        self._name = name or self.__class__.__name__
+        self._requests_processed = 0
+        self._sessions = 0
+        self._max_requests = max_requests
+        self._logger = logger
 
     @property
     def sessions(self):
@@ -487,14 +493,17 @@ class Producer(EventHandler):
         '''
         return self._requests_processed
 
-    def create_protocol(self):
+    def create_protocol(self, **kw):
         '''Create a new protocol via the :meth:`protocol_factory`
 
         This method increase the count of :attr:`sessions` and build
         the protocol passing ``self`` as the producer.
         '''
-        self._sessions = session = self._sessions + 1
-        return self.protocol_factory(session=session, producer=self)
+        self._sessions = self._sessions + 1
+        kw['session'] = self.sessions
+        kw['producer'] = self
+        kw['loop'] = self._loop
+        return self.protocol_factory(**kw)
 
     def build_consumer(self, consumer_factory):
         '''Build a consumer for a protocol.
@@ -504,13 +513,13 @@ class Producer(EventHandler):
 
         :param consumer_factory: consumer factory to use.
         '''
-        consumer = consumer_factory()
+        consumer = consumer_factory(loop=self._loop)
         consumer.copy_many_times_events(self)
         return consumer
 
 
 class TcpServer(Producer):
-    '''A :class:`Producer` of server :class:`Connection` for TCP servers.
+    '''A :class:`.Producer` of server :class:`Connection` for TCP servers.
 
     .. attribute:: _server
 
@@ -525,14 +534,13 @@ class TcpServer(Producer):
     _started = None
 
     def __init__(self, protocol_factory, loop, address=None,
-                 name=None, sockets=None, max_connections=None,
-                 keep_alive=None):
-        super(TcpServer, self).__init__(loop)
-        self.protocol_factory = protocol_factory
-        self._name = name or self.__class__.__name__
+                 name=None, sockets=None, max_requests=None,
+                 keep_alive=None, logger=None):
+        super(TcpServer, self).__init__(loop, protocol_factory, name=name,
+                                        max_requests=max_requests,
+                                        logger=logger)
         self._params = {'address': address, 'sockets': sockets}
-        self._max_connections = max_connections
-        self._keep_alive = keep_alive
+        self._keep_alive = max(keep_alive or 0, 0)
         self._concurrent_connections = set()
 
     def __repr__(self):
@@ -569,21 +577,21 @@ class TcpServer(Producer):
                 if sockets:
                     server = None
                     for sock in sockets:
-                        srv = yield create_server(self.create_protocol,
-                                                  sock=sock,
-                                                  backlog=backlog,
-                                                  ssl=sslcontext)
+                        srv = yield From(create_server(self.create_protocol,
+                                                       sock=sock,
+                                                       backlog=backlog,
+                                                       ssl=sslcontext))
                         if server:
                             server.sockets.extend(srv.sockets)
                         else:
                             server = srv
                 else:
                     if isinstance(address, tuple):
-                        server = yield create_server(self.create_protocol,
-                                                     host=address[0],
-                                                     port=address[1],
-                                                     backlog=backlog,
-                                                     ssl=sslcontext)
+                        server = yield From(create_server(self.create_protocol,
+                                                          host=address[0],
+                                                          port=address[1],
+                                                          backlog=backlog,
+                                                          ssl=sslcontext))
                     else:
                         raise NotImplementedError
                 self._server = server
@@ -612,8 +620,8 @@ class TcpServer(Producer):
             if self._server:
                 server, self._server = self._server, None
                 server.close()
-                yield None
-                yield self._close_connections()
+                yield From(None)
+                yield From(self._close_connections())
             self.fire_event('stop')
 
     def info(self):
@@ -623,7 +631,7 @@ class TcpServer(Producer):
                   'python_version': sys.version,
                   'uptime_in_seconds': up,
                   'sockets': sockets,
-                  'max_connections': self._max_connections,
+                  'max_requests': self._max_requests,
                   'keep_alive': self._keep_alive}
         clients = {'processed_clients': self._sessions,
                    'connected_clients': len(self._concurrent_connections),
@@ -638,16 +646,14 @@ class TcpServer(Producer):
     def create_protocol(self):
         '''Override :meth:`Producer.create_protocol`.
         '''
-        self._sessions = session = self._sessions + 1
-        protocol = self.protocol_factory(session=session,
-                                         producer=self,
-                                         timeout=self._keep_alive)
+        protocol = super(TcpServer, self).create_protocol(
+            timeout=self._keep_alive)
         protocol.bind_event('connection_made', self._connection_made)
         protocol.bind_event('connection_lost', self._connection_lost)
-        if (self._server and self._max_connections and
-                session >= self._max_connections):
+        if (self._server and self._max_requests and
+                self._sessions >= self._max_requests):
             self.logger.info('Reached maximum number of connections %s. '
-                             'Stop serving.' % self._max_connections)
+                             'Stop serving.' % self._max_requests)
             self.close()
         return protocol
 
@@ -680,8 +686,8 @@ class TcpServer(Producer):
             return multi_async(all)
 
 
-class DatagramServer(EventHandler):
-    '''An :class:`.EventHandler` for serving UDP sockets.
+class DatagramServer(Producer):
+    '''An :class:`.Producer` for serving UDP sockets.
 
     .. attribute:: _transports
 
@@ -696,13 +702,11 @@ class DatagramServer(EventHandler):
     MANY_TIMES_EVENTS = ('pre_request', 'post_request')
 
     def __init__(self, protocol_factory, loop=None, address=None,
-                 name=None, sockets=None, max_requests=None):
-        self._loop = loop or get_event_loop()
-        super(DatagramServer, self).__init__(self._loop)
-        self.protocol_factory = protocol_factory
-        self._max_requests = max_requests
-        self._requests_processed = 0
-        self._name = name or self.__class__.__name__
+                 name=None, sockets=None, max_requests=None,
+                 logger=None):
+        super(DatagramServer, self).__init__(loop, protocol_factory, name=name,
+                                             max_requests=max_requests,
+                                             logger=logger)
         self._params = {'address': address, 'sockets': sockets}
 
     @task
@@ -723,8 +727,9 @@ class DatagramServer(EventHandler):
                         proto = self.create_protocol()
                         transports.append(transport(self._loop, proto))
                 else:
-                    transport, _ = yield self._loop.create_datagram_endpoint(
-                        self.protocol_factory, local_addr=adress)
+                    transport, _ = yield From(
+                        self._loop.create_datagram_endpoint(
+                            self.protocol_factory, local_addr=address))
                     transports.append(transport)
                 self._transports = transports
                 self._started = self._loop.time()
@@ -738,7 +743,7 @@ class DatagramServer(EventHandler):
                 self.fire_event('start', exc=exc)
                 self.fire_event('stop')
 
-    @in_loop
+    @task
     def close(self):
         '''Stop serving the :attr:`.Server.sockets` and close all
         concurrent connections.
@@ -765,8 +770,3 @@ class DatagramServer(EventHandler):
                     'address': format_address(transport._sock.getsockname())})
         return {'server': server,
                 'clients': clients}
-
-    def create_protocol(self):
-        '''Override :meth:`Producer.create_protocol`.
-        '''
-        return self.protocol_factory(producer=self)
