@@ -30,7 +30,7 @@ from . import system
 from .internet import parse_address
 from .importer import import_system_file
 from .httpurl import HttpParser as PyHttpParser
-from .log import configured_logger
+from .log import configured_logger, get_level
 from .pep import to_bytes, iteritems, native_str, pickle
 
 
@@ -77,6 +77,14 @@ def ordered_settings():
         yield KNOWN_SETTINGS[name]
 
 
+def valid_config_value(val):
+    try:
+        pickle.loads(pickle.dumps(val))
+        return True
+    except Exception:
+        return False
+
+
 class Config(object):
     '''A dictionary-like container of :class:`Setting` parameters for
     fine tuning pulsar servers.
@@ -109,8 +117,8 @@ class Config(object):
 
     .. attribute:: params
 
-        Dictionary of additional parameters which cannot be parsed in the
-        command line.
+        Dictionary of additional parameters which cannot be parsed on the
+        command line
     '''
     script = None
     application = None
@@ -121,6 +129,7 @@ class Config(object):
                  exclude=None, settings=None, prefix=None,
                  name=None, log_name=None, **params):
         self.settings = {} if settings is None else settings
+        self.params = {}
         self.name = name
         self.log_name = log_name
         self.prefix = prefix
@@ -129,7 +138,6 @@ class Config(object):
         self.apps = set(apps or ())
         if settings is None:
             self.update_settings()
-        self.params = {}
         self.description = description or 'Pulsar server'
         self.epilog = epilog or 'Have fun!'
         self.version = version or __version__
@@ -269,7 +277,8 @@ class Config(object):
                 val = getattr(mod, k)
                 # add unknown names to list
                 if kl not in self.settings:
-                    unknowns.append((k, val))
+                    if valid_config_value(val):
+                        unknowns.append((k, val))
                 else:
                     self.set(kl, val)
         return unknowns
@@ -349,43 +358,56 @@ class Config(object):
         '''Configured logger.
         '''
         loggers = {}
+        internal_level = get_level('warning')
         loghandlers = self.loghandlers
-        if not name and self.log_name:
-            name = self.log_name
-        else:
-            name = 'pulsar.%s' % (name or self.name)
-        default_loglevel = None
+        # base name is always pulsar
+        basename = 'pulsar'
+        # the namespace which uses the default log level
+        # (the one without namespace prefix)
+        defname = self.log_name or basename
+        # the namespace name for this config
+        name = name or self.name
+        if name:
+            name = '%s.%s' % (basename, name)
+        #
+        namespaces = {}
+
         for loglevel in self.loglevel or ():
             bits = loglevel.split('.')
-            loglevel = bits[-1]
-            if len(bits) > 1:
-                namespace = '.'.join(bits[:-1])
-            else:
-                namespace = name
+            namespaces['.'.join(bits[:-1]) or defname] = bits[-1]
+
+        if defname not in namespaces:
+            namespaces[defname] = self.settings['loglevel'].default[0]
+
+        deflevel = get_level(namespaces[defname])
+        internal_level = max(deflevel, internal_level) if deflevel else 0
+        for namespace in (basename, 'asyncio'):
+            if namespace not in namespaces:
+                namespaces[namespace] = internal_level
+        # Trollius same as asyncio
+        namespaces['trollius'] = namespaces['asyncio']
+
+        if name and name not in namespaces:
+            namespaces[name] = namespaces[basename]
+
+        for namespace in sorted(namespaces):
             if namespace in loggers:
                 continue
+            if self.daemon:     # pragma    nocover
+                handlers = []
+                for hnd in loghandlers:
+                    if hnd != 'console':
+                        handlers.append(hnd)
+                if not handlers:
+                    handlers.append('file')
+                loghandlers = handlers
             logger = configured_logger(namespace,
                                        config=self.logconfig,
-                                       level=loglevel,
-                                       handlers=loghandlers)
+                                       level=namespaces[namespace],
+                                       handlers=loghandlers,
+                                       rootlevel=internal_level)
             loggers[namespace] = logger
-            if namespace == 'pulsar' or default_loglevel is None:
-                default_loglevel = logger.level
-
-        default_loglevel = asyncio_loglevel = default_loglevel or 0
-        if asyncio_loglevel:
-            asyncio_loglevel = max(asyncio_loglevel,
-                                   logging._checkLevel('WARNING'))
-        for namespace, loglevel in (('pulsar', default_loglevel),
-                                    (name, default_loglevel),
-                                    ('asyncio', asyncio_loglevel),
-                                    ('trollius', asyncio_loglevel)):
-            if namespace not in loggers:
-                loggers[namespace] = configured_logger(namespace,
-                                                       config=self.logconfig,
-                                                       level=loglevel,
-                                                       handlers=loghandlers)
-        return loggers[name]
+        return loggers.get(name)
 
     def __copy__(self):
         return self.copy()
@@ -400,12 +422,13 @@ class Config(object):
             setting = s().copy(name=self.name, prefix=self.prefix)
             if setting.name in self.settings:
                 continue
-            if setting.name not in self.include:
-                if setting.name in self.exclude:
-                    continue    # setting name in exclude set
-                if setting.app and setting.app not in self.apps:
-                    continue    # the setting is for an app not in the apps set
-            self.settings[setting.name] = setting
+            if setting.app and setting.app not in self.apps:
+                continue    # the setting is for an app not in the apps set
+            if ((self.include and setting.name not in self.include) or
+                    setting.name in self.exclude):
+                self.params[setting.name] = setting.get()
+            else:
+                self.settings[setting.name] = setting
 
     def _get(self, name, default=None):
         if name not in self.settings:
@@ -780,7 +803,7 @@ class Debug(Global):
         Turn on debugging in the server.
 
         Set the log level to debug, limits the number of worker processes
-        to 1 and changes some error handling that's sent to clients.
+        to 1, set asyncio debug flag.
         """
 
 
@@ -798,6 +821,19 @@ class Daemon(Global):
         """
 
 
+class Reload(Global):
+    name = "reload"
+    flags = ["--reload"]
+    validator = validate_bool
+    action = "store_true"
+    default = False
+    desc = """\
+        Auto reload modules when changes occurs.
+
+        Useful during development.
+        """
+
+
 class Noisy(Global):
     name = "noisy"
     flags = ["--noisy"]
