@@ -8,13 +8,19 @@ class PulsarException(Exception):
 
 
 class MonitorStarted(PulsarException):
-    pass
+    exit_code = 0
 
 
 class ImproperlyConfigured(PulsarException):
     '''A :class:`PulsarException` raised when an inconsistent configuration
-has occured.'''
-    pass
+    has occured.
+
+    .. attribute:: exit_code
+
+        the exit code when rising this exception is set to 2. This will
+        cause pulsar to log the error rather than the full stack trace.
+    '''
+    exit_code = 2
 
 
 class CommandError(PulsarException):
@@ -29,8 +35,11 @@ class CommandNotFound(CommandError):
 
 
 class ProtocolError(PulsarException):
-    '''Raised when the protocol encounter unexpected data. It will close
-the socket connection.'''
+    '''A :class:`PulsarException` raised when the protocol encounter
+    unexpected data.
+
+    It will close the socket connection
+    '''
     status_code = None
 
     def ProtocolError(self, msg=None, status_code=None):
@@ -38,14 +47,6 @@ the socket connection.'''
         self.status_code = status_code
 
 
-class TooManyConsecutiveWrite(PulsarException):
-    '''Raise when too many consecutive writes are attempted.'''
-
-
-class TooManyConnections(PulsarException):
-    '''Raised when there are too many concurrent connections.'''
-
-
 class EventAlreadyRegistered(PulsarException):
     pass
 
@@ -60,7 +61,7 @@ class HaltServer(BaseException):
 
     When ``exit_code`` is greater than 1, it is considered an expected
     failure and therefore the full stack trace is not logged.'''
-    def __init__(self, reason='Exiting server.', exit_code=2):
+    def __init__(self, reason='Exiting server.', exit_code=3):
         super(HaltServer, self).__init__(reason)
         self.exit_code = exit_code
 
@@ -79,18 +80,18 @@ class SSLError(HTTPError):
 class HttpException(HTTPError):
     '''The base class of all ``HTTP`` server exceptions
 
-Introduces the following attributes:
+    Introduces the following attributes:
 
-.. attribute:: status
+    .. attribute:: status
 
-    The numeric status code for the exception (ex 500 for server error).
+        The numeric status code for the exception (ex 500 for server error).
 
-    Default: ``500``.
+        Default: ``500``.
 
-.. attribute:: headers
+    .. attribute:: headers
 
-    Additional headers to add to the client response.
-'''
+        Additional headers to add to the client response.
+    '''
     status = 500
 
     def __init__(self, msg='', status=None, handler=None, strict=False,
@@ -147,8 +148,3 @@ class Http404(HttpException):
 class MethodNotAllowed(HttpException):
     '''An :class:`HttpException` with default ``405`` status code.'''
     status = 405
-
-
-class ContentNotAccepted(HttpException):
-    '''An :class:`HttpException` with default ``406`` status code.'''
-    status = 406
