@@ -1281,63 +1281,29 @@ def create_cookie(name, value, **kwargs):
     return Cookie(**result)
 
 
-def cookiejar_from_dict(cookie_dict, cookiejar=None):
+def cookiejar_from_dict(*cookie_dicts):
     """Returns a CookieJar from a key/value dictionary.
 
     :param cookie_dict: Dict of key/values to insert into CookieJar.
     """
-    if not isinstance(cookie_dict, CookieJar):
-        if cookiejar is None:
-            cookiejar = CookieJar()
-        if cookie_dict is not None:
+    jars = []
+    cookie_dicts = tuple((d for d in cookie_dicts if d))
+    if len(cookie_dicts) == 1 and isinstance(cookie_dicts[0], CookieJar):
+        return cookie_dicts[0]
+    cookiejar = CookieJar()
+    for cookie_dict in cookie_dicts:
+        if isinstance(cookie_dict, CookieJar):
+            for cookie in cookie_dict:
+                cookiejar.set_cookie(cookie)
+        else:
             for name in cookie_dict:
                 cookiejar.set_cookie(create_cookie(name, cookie_dict[name]))
-        return cookiejar
-    else:
-        return cookie_dict
+    return cookiejar
 
 
 cc_delim_re = re.compile(r'\s*,\s*')
 
 
-class accept_content_type(object):
-
-    def __init__(self, values=None):
-        self._all = {}
-        self.update(values)
-
-    def update(self, values):
-        if values:
-            all = self._all
-            accept_headers = cc_delim_re.split(values)
-            for h in accept_headers:
-                v = h.split('/')
-                if len(v) == 2:
-                    a, b = v
-                    if a in all:
-                        all[a].append(b)
-                    else:
-                        all[a] = [b]
-
-    def __contains__(self, content_type):
-        all = self._all
-        if not all:
-            # If no Accept header field is present, then it is assumed that the
-            # client accepts all media types.
-            return True
-        a, b = content_type.split('/')
-        if a in all:
-            all = all[a]
-            if '*' in all:
-                return True
-            else:
-                return b in all
-        elif '*' in all:
-            return True
-        else:
-            return False
-
-
 def patch_vary_headers(response, newheaders):
     """\
 Adds (or updates) the "Vary" header in the given HttpResponse object.
