@@ -8,7 +8,7 @@ from time import time
 from threading import Lock
 from functools import wraps
 
-from .system import current_process
+from .system import current_process, platform
 
 
 from .pep import force_native_str
@@ -73,6 +73,16 @@ LOGGING_CONFIG = {
 }
 
 
+if not platform.is_appengine:
+
+    def file_handler(**kw):
+        return logging.FileHandler('pulsar.log', **kw)
+
+    LOGGING_CONFIG['handlers']['file'] = {
+        '()': file_handler,
+        'formatter': 'verbose'}
+
+
 def update_config(config, c):
     for name in ('handlers', 'formatters', 'filters', 'loggers', 'root'):
         if name in c:
@@ -214,7 +224,8 @@ class Silence(logging.Handler):
         pass
 
 
-def configured_logger(name, config=None, level=None, handlers=None):
+def configured_logger(name, config=None, level=None, handlers=None,
+                      rootlevel=None):
     '''Configured logger.
     '''
     with process_global('lock'):
@@ -235,17 +246,8 @@ def configured_logger(name, config=None, level=None, handlers=None):
             logconfig.pop('loggers', None)
             logconfig.pop('root', None)
 
-        if level is None:
-            level = logging.NOTSET
-        else:
-            try:
-                level = int(level)
-            except ValueError:
-                lv = str(level).upper()
-                try:
-                    level = logging._checkLevel(lv)
-                except ValueError:
-                    level = logging.NOTSET
+        level = get_level(level)
+        rootlevel = get_level(rootlevel or level)
         # No loggers configured. This means no logconfig setting
         # parameter was used. Set up the root logger with default
         # loggers
@@ -261,13 +263,38 @@ def configured_logger(name, config=None, level=None, handlers=None):
         logconfig['loggers'][name] = l
         #
         if not original.get('root'):
-            logconfig['root'] = {'handlers': handlers,
-                                 'level': level}
+            logconfig['root'] = {'handlers': handlers, 'level': rootlevel}
         if logconfig:
             dictConfig(logconfig)
         return logging.getLogger(name)
 
 
+def get_level(level):
+    try:
+        return int(level)
+    except TypeError:
+        return logging.NOTSET
+    except ValueError:
+        lv = str(level).upper()
+        try:
+            return logging._checkLevel(lv)
+        except ValueError:
+            return logging.NOTSET
+
+
+def logger_fds():
+    logger = logging.getLogger()
+    loggers = set(logger.manager.loggerDict.values())
+    loggers.add(logger)
+    fds = set()
+    for logger in loggers:
+        for hnd in getattr(logger, 'handlers', ()):
+            try:
+                fds.add(hnd.stream.fileno())
+            except AttributeError:
+                pass
+    return fds
+
 WHITE = 37
 COLOURS = {'red': 31,
            'green': 32,
@@ -304,7 +331,7 @@ class ColoredStream(logging.StreamHandler):   # pragma    nocover
     def color(self, record):
         text = self.format(record)
         file = self.stream
-        if file.isatty() or True:
+        if hasattr(file, 'isatty') and file.isatty():
             colour = self.COLORS.get(record.levelname)
             code = COLOURS.get(colour, WHITE)
             if win32:
